@@ -1,12 +1,13 @@
 import logging
 from dataclasses import dataclass, field, InitVar
-from typing import Dict, List, Tuple
+from typing import Dict, List, Tuple, Optional
 
 from google.protobuf.internal import enum_type_wrapper
 
 from .proto.common_types import common_types_pb2
 from .qmpc_server import QMPCServer
 from .share import Share
+from .utils.restore import restore
 from .utils.parse_csv import (parse, parse_csv, parse_csv_to_bitvector,
                               parse_to_bitvector)
 
@@ -18,6 +19,7 @@ ComputationMethod: enum_type_wrapper.EnumTypeWrapper \
     = common_types_pb2.ComputationMethod
 PredictMethod: enum_type_wrapper.EnumTypeWrapper \
     = common_types_pb2.PredictMethod
+JobErrorInfo = common_types_pb2.JobErrorInfo
 
 
 @dataclass(frozen=True)
@@ -178,10 +180,12 @@ class QMPC:
             ComputationMethod.Value("COMPUTATION_METHOD_JOIN_TABLE"),
             join_order, (join_order[2], []))
 
-    def get_computation_result(self, job_id: str) -> Dict:
+    def get_computation_result(self, job_id: str,
+                               path: Optional[str] = None) -> Dict:
         logger.info("get_computation_result request. "
-                    f"[job_id]={job_id}")
-        return self.__qmpc_server.get_computation_result(job_id)
+                    f"[job_id]={job_id} "
+                    f"[path]={path}")
+        return self.__qmpc_server.get_computation_result(job_id, path)
 
     def send_model_params(self, params: list,
                           piece_size: int = 1_000_000) -> Dict:
@@ -262,6 +266,12 @@ class QMPC:
                     f"[job_id]={job_id}")
         return self.__qmpc_server.get_elapsed_time(job_id)
 
+    def restore(self, job_id: str, path: str):
+        logger.info("restore request. "
+                    f"[job_id]={job_id} "
+                    f"[path]={path}")
+        return restore(job_id, path, self.__party_size)
+
     @staticmethod
     def set_log_level(level: int):
         logger.setLevel(level)
