@@ -2,6 +2,9 @@
 ローカルでhttpsサーバを3台立てており，
 yieldのタイミングで全てのTestが開始
 """
+import glob
+import os
+
 import pytest
 
 from .local_server import serve
@@ -17,6 +20,9 @@ def run_server1():
     yield
 
     """ test end """
+    for file in glob.glob("./tests/unit_tests/dim1-uuid*"):
+        os.remove(file)
+
     server.stop(0)
 
 
